import java.applet.Applet;
import java.awt.*;
import java.awt.event.*;
import java.applet.*;
import java.awt.geom.*;
import java.math.*;


public class DialConversions {


    /**Conversion routines*/


    /**This is really stereographic projection
       from the upper half plane model of the hyperbolic  
       plane to the Poincare disk model.*/
    
    public static Complex realToComplex(double s) {
       Complex z=new Complex(s,0);
       Complex z1=new Complex(z.x,z.y-1);
       Complex z2=new Complex(z.x,z.y+1);
       Complex z3=Complex.divide(z1,z2);
       return z3;
    }



    /**This gets the matrix which converts aspect ratio to slope*/
    
    public static double[][] aspectToSlopeMatrix(Segment[] S) {
	double m1=S[0].slope();
	double b1=S[0].yIntercept();
	double m2=S[1].slope();
	double b2=S[1].yIntercept();
	double m3=S[2].slope();
	double b3=S[2].yIntercept();
	double m4=S[3].slope();
	double b4=S[3].yIntercept();
        double A=b3*m1-b4*m1-b3*m2+b4*m2-b1*m3+b2*m3+b1*m4-b2*m4;
        double B=b3*m1*m2-b4*m1*m2-b2*m1*m3+b4*m1*m3+b1*m2*m4-b3*m2*m4-b1*m3*m4+b2*m3*m4;
        double C=-(b2*m1*m3)+b4*m1*m3+b1*m2*m3-b4*m2*m3+b2*m1*m4-b3*m1*m4-b1*m2*m4+b3*m2*m4;
        double D=-(b2*m1)+b3*m1+b1*m2-b4*m2-b1*m3+b4*m3+b2*m4-b3*m4;
        double[][] m={{A,B},{C,D}};
        return m;
    }

    public static double[][] aspectToSlopeMatrixInverse(Segment[] S) {
	double[][] m=aspectToSlopeMatrix(S);
	double[][] mm={{m[1][1],-m[0][1]},{-m[1][0],m[0][0]}};
	return mm;
    }

    /**This is the Mobius transformation that carries the
       aspect ratio of the rectangle to its slope.*/
    
    public static double aspectToSlope(Segment[] S,double a) {
       double[][] m=aspectToSlopeMatrix(S);
       double s=(m[0][0]*a+m[0][1])/(m[1][0]*a+m[1][1]);
       return s;
    }

    /**This is the inverse Mobius transformation which carries
       the slope of the rectangle to its aspect ratio*/

    public static double slopeToAspect(Segment[] S,double s) {
       double[][] m=aspectToSlopeMatrixInverse(S);
       double a=(m[0][0]*s+m[0][1])/(m[1][0]*s+m[1][1]);
       return a;
    }



    public static Complex act(double[][] m,Complex A) {
	Complex[][] M=new Complex[2][2];
	for(int i=0;i<2;++i) {
	    for(int j=0;j<2;++j) {
		M[i][j]=new Complex(m[i][j],0);
	    }
	}
	Complex M1=Complex.plus(Complex.times(M[0][0],A),M[0][1]);
	Complex M2=Complex.plus(Complex.times(M[1][0],A),M[1][1]);
	return Complex.divide(M1,M2);
    }

}




